/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the "License").  You may not use this file except
 * in compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://jwsdp.dev.java.net/CDDLv1.0.html
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * HEADER in each file and include the License file at
 * https://jwsdp.dev.java.net/CDDLv1.0.html  If applicable,
 * add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your
 * own identifying information: Portions Copyright [yyyy]
 * [name of copyright owner]
 */
/*
 * Copyright (c) 2006 Sun Microsystems, Inc.
 * All rights reserved.
 */
package simple.client;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPBinding;
import javax.xml.namespace.QName;
import java.io.*;
import java.util.Iterator;
import org.w3c.dom.*;
import javax.xml.soap.*;
import javax.xml.parsers.*;
import javax.xml.transform.dom.*;

import test.parse.*;

public class PingServiceClients22{

    public static void main(String[] args) {
            /* Uncomment to run against Local Server*/
        String testStatus = "notset";
        IPingService stub = null;
        
        String requestTestStatus = null , responseTestStatus = null;
        String testLogFile = null;
        String[] mesgString = null;
        String runOutputFilePath = null;
        LogFileProcessor logFileProcessor = null;
        
        try {
            
            PingService service = new PingService();
            stub = service.getPingPort();

            // use static stubs to override endpoint property of WSDL
            String serviceHost = System.getProperty("endpoint.host");
            String servicePort = System.getProperty("endpoint.port");
            String serviceURLFragment = System.getProperty("service.url");
            String serviceURL =
                "https://" + serviceHost + ":" + servicePort + serviceURLFragment;

            //System.out.println("Service URL=" + serviceURL);
            
            ((BindingProvider)stub).getRequestContext().
                put(javax.xml.ws.BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
            
        } catch (Exception wsEx) {
            System.out.println("Error in creating jax-ws webservice");
            wsEx.printStackTrace();
        }
        
        try {
            //System.out.println("Testing WSSecurityPolicy!");
            String greeting = stub.ping("Hello!");
            //System.out.println("greeting was : "+greeting);
        } catch (Exception noSoapEx) {
            System.out.println("Error : SOAP request NOT sent ");
            noSoapEx.printStackTrace();
            //write fail message to output file
            logFileProcessor = new LogFileProcessor(1);
            runOutputFilePath = logFileProcessor.createXtestOutputFile();
            try{
                FileWriter fw = new FileWriter(runOutputFilePath);
                String str ="FAILED"+ "\n" + "SOAP request NOT sent ; \n" + noSoapEx.getMessage() + "\n";
                StackTraceElement[] stackTraceArray = noSoapEx.getStackTrace();
                for(int i=0; i< stackTraceArray.length ; i++) {
                    str += stackTraceArray[i].toString()+"\n";
                }
                str += "CHECK Runtime Log for details";
                fw.write(str);
                //noSoapEx.printStackTrace((PrintWriter)((Writer)fw));
                fw.close();
            }catch(Exception fileWriteEx){
                System.out.println(fileWriteEx.getMessage());
            }
            return;
        }
        
        try {
            logFileProcessor = new LogFileProcessor(1);
            testLogFile = logFileProcessor.getTestRunLogFileName();            
            //System.out.println("*** *** testLogFile name : "+ testLogFile.toString() + "*** ***");
        } catch (Exception testLogFileEx) {
            System.out.println("Error in accessing Test Run Log File");
            testLogFileEx.printStackTrace();
        }
        
        try {
            mesgString = new String[2];                        
            mesgString = logFileProcessor.separateLog(testLogFile);           
        }catch(Exception logParseEx) {
            System.out.println("Error : Could not parse test run log file");
            logParseEx.printStackTrace();
        }
        
        try {
            requestTestStatus = checkRequestMesg((File[])logFileProcessor.getRequestFileValue());
            //System.out.println("requestTestStatus : " + requestTestStatus);
        }catch (Exception checkRequestFileEx) {
            System.out.println("Error in checking Request Message File");
            checkRequestFileEx.printStackTrace();
        }
            
        try {
            responseTestStatus = checkResponseMesg((File[])logFileProcessor.getResponseFileValue());
            //System.out.println("responseTestStatus : "+responseTestStatus);
        }catch (Exception checkResponseFileEx) {
            System.out.println("Error in checking Response Message File");
            checkResponseFileEx.printStackTrace();
        }
        
        //Create log file and dump the result        
        try{
            runOutputFilePath = logFileProcessor.createXtestOutputFile();
            FileWriter fw = new FileWriter(runOutputFilePath);
            String str = null;
            if (requestTestStatus.startsWith("PASSED") && responseTestStatus.startsWith("PASSED")) {
                str ="PASSED"+"\n" + 
                     "Request message validation : " + requestTestStatus.replaceFirst("PASSED","") +
                     "Response message validation : " + responseTestStatus.replaceFirst("PASSED","");
            } else {
                str = (requestTestStatus.startsWith("FAILED")) ? requestTestStatus.replaceFirst("FAILED","") : responseTestStatus.replaceFirst("FAILED","");
                str = "FAILED"+"\n"+str;
            }
            fw.write(str);
            fw.close();
        }catch(Exception ex){
            System.out.println(ex.getMessage());
        }        
    }
    
    public static String checkRequestMesg(File[] requestFile) {
        System.out.println("\n\nIn checkRequestMesg :  checking Request mesg");
        if(requestFile == null){
            System.out.println("Error opening Request File: START checkRequestMesg");
            return "FAILED : \nError opening Request file";
        }
        String checkStatus = "FAILED" + "\n Unexpectedly during checking request soap message ; see test run logs";
        try {
            for(int i=0 ; i<requestFile.length ; i++) {
        
                try {
                    System.out.println("\n\tRequest message file name : " + requestFile[i].getName());
                } catch (Exception fileException) {
                    System.out.println("Error in opening request message file");
                    fileException.printStackTrace();
                    return "FAILED : \nError in opening request message file:Check run time log";
                }
        
                try {
                    MessageFactory mFactory = MessageFactory.newInstance();
                    SOAPMessage message = mFactory.createMessage();
            
                    //Put the requestFile = request soap message xml into the SOAPPart of the above created 'message'
                    DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
                    dbFactory.setNamespaceAware(true);
                    DocumentBuilder dbuilder = dbFactory.newDocumentBuilder();
                    Document document = dbuilder.parse(requestFile[i]);
                    DOMSource domSource = new DOMSource(document);  
        
                    SOAPPart soapPart = message.getSOAPPart();
                    soapPart.setContent(domSource);
            
                    org.w3c.dom.Node tstampNode = null, usernameTokenNode = null, signatureNode = null, signedInfoNode = null , keyInfoNode = null, binarySecTokenNode = null;
                    String tstampID = null, usernameTokenID = null , binarySecTokenID = null;
                    //Get SOAP header
                    SOAPHeader soapHeader = message.getSOAPHeader();
                    Iterator headerIterator = soapHeader.examineAllHeaderElements();
                    while(headerIterator.hasNext()){
                        org.w3c.dom.Node soapHeaderNode = (org.w3c.dom.Node) headerIterator.next();
                        System.out.println("soapHeaderNode name : " + soapHeaderNode.getNodeName());
                        if(soapHeaderNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE && 
                                soapHeaderNode.getLocalName().equals("Security") && 
                                soapHeaderNode.getNamespaceURI().equals("http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd")) {
                            org.w3c.dom.NodeList securityHeaderChildren = soapHeaderNode.getChildNodes();
                            for(int j = 0 ; j < securityHeaderChildren.getLength() ; j++) {
                                if(securityHeaderChildren.item(j).getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
                                    if(securityHeaderChildren.item(j).getLocalName().equals("Timestamp") &&
                                            securityHeaderChildren.item(j).getNamespaceURI().equals("http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd")) {
                                        tstampNode = securityHeaderChildren.item(j);
                                        tstampID = tstampNode.getAttributes().getNamedItem("wsu:Id").getNodeValue();
                                        System.out.println("Found Timestamp : " + tstampNode.getNodeName() + " with wsu:Id=" +tstampID);
                                        checkStatus = "PASSED" + "\nTransportBinding with Timestamp element and Strict layout";
                                        //return checkStatus;
                                    }                                               
                                }
                            }
                        }
                    }
            
                }catch(Exception soapException) {
                    soapException.printStackTrace();
                    return "FAILED" +"\n" + soapException.getMessage();
                }
        
                //return "FAILED" + "\n Unexpectedly during checking request soap message ; see test run logs";
            }
        } catch(Exception fileArrayEx){
            System.out.println("Error in the File array containing the request/response xmls messages");
            fileArrayEx.printStackTrace();
            return "FAILED" + "\n Unexpectedly during checking request soap message ; see test run logs";
        }
        return "PASSED" +"\n Request message Validation passed";
    }
    
     public static String checkResponseMesg(File[] responseFile) {
        System.out.println("\n\nIn checkResponseMesg :  checking Response mesg");
        if(responseFile == null){
            return "FAILED : \nError opening Response file";
        }
        String checkStatus = "FAILED" + "\n Unexpectedly during checking request soap message ; see test run logs";
        try {
            for (int i = 0 ; i < responseFile.length ; i++) {
                try {
                    System.out.println("\n\tResponse message file name : " + responseFile[i].getName());
                } catch (Exception fileException) {
                    System.out.println("Error in opening response message file");
                    fileException.printStackTrace();
                    return "FAILED : \nError in opening response message file:Check tests";
                }
                //Use SAAJ APIs to read the response soap message and obtain fault string
                try {
                    MessageFactory mFactory = MessageFactory.newInstance();
                    SOAPMessage message = mFactory.createMessage();
        
                    //Put the responseFile = response soap message xml into the SOAPPart of the above created 'message'
                    DocumentBuilderFactory dbFactory =  DocumentBuilderFactory.newInstance();
                    dbFactory.setNamespaceAware(true);
                    DocumentBuilder dbuilder = dbFactory.newDocumentBuilder();
                    Document document = dbuilder.parse(responseFile[i]);
                    DOMSource domSource = new DOMSource(document);
        
                    SOAPPart soapPart = message.getSOAPPart();
                    soapPart.setContent(domSource);
            
                    //Check for SOAPFault in response message:
                    SOAPBody soapBody = message.getSOAPBody();
                    if (soapBody.hasFault()) {
                        System.out.println("Response message has SOAPFault");
                        SOAPFault soapFault = soapBody.getFault();
                        System.out.println("FaultCode : " + soapFault.getFaultCode());
                        System.out.println("FaultString : " + soapFault.getFaultString());
                
                        return "FAILED : " +"\n" + "SOAP Fault returned " + "\n  FaultCode : " +soapFault.getFaultCode() +
                            "\n  FaultString : "+soapFault.getFaultString();
                    }else {
                        checkStatus = "PASSED" + "\nNo SOAP Fault in response message \nwsu:Timestamp element " + 
                                                "present in response as desired ";
                    }
                }catch(Exception soapException){
                    soapException.printStackTrace();
                    StackTraceElement[] stackTraceArray = soapException.getStackTrace();
                    String str = soapException.getMessage() + "\n";
                    for(int j=0; j < stackTraceArray.length ; j++) {
                        str += stackTraceArray[j].toString()+"\n";
                    }
                    str += "CHECK Runtime Log for details";
                    return "FAILED" + "\n" + str;
                }
        
            }
        }catch(Exception genEx){
            System.out.println("Error checking response xml messages");
            genEx.printStackTrace();                    
        }
        return checkStatus;
    }
    
}
